-module(wsTest).

-compile([export_all]).

-define(Line, <<"GET /test/tttt HTTP/1.1\r\n">>).

test1(Rn) ->
   parseRequestLine(?Line, Rn).

test2() ->
   erlang:decode_packet(http_bin, ?Line, []).

parseRequestLine(Data, Rn) ->
   case binary:split(Data, Rn) of
      [Data] ->
         not_enough_data;
      [Line, Rest] ->
         case binary:split(Line, <<" ">>, [global]) of
            [Method, RawPath, V] ->
               {Method, RawPath, V, Rest};
            _ ->
               {error, request_line}
         end
   end.


%% <<"Content-Type: application/json; charset=utf-8">>,
-spec request(boolean(), body(), method(), host(), binary(), path(), headers()) -> iolist().
request(true, undefined, Method, Host, _DbName, Path, Headers) ->
   [
      Method, <<"/_db/_system">>, Path, <<" HTTP/1.1\r\nHost: ">>, Host,
      <<"\r\nContent-Type: application/json; charset=utf-8\r\nContent-Length: 0\r\n">>,
      spellHeaders(Headers), <<"\r\n">>
   ];
request(false, undefined, Method, Host, DbName, Path, Headers) ->
   [
      Method, DbName, Path, <<" HTTP/1.1\r\nHost: ">>, Host,
      <<"\r\nContent-Type: application/json; charset=utf-8\r\nContent-Length: 0\r\n">>,
      spellHeaders(Headers), <<"\r\n">>
   ];
request(false, Body, Method, Host, DbName, Path, Headers) ->
   ContentLength = integer_to_binary(iolist_size(Body)),
   NewHeaders = [{<<"Content-Length">>, ContentLength} | Headers],
   [
      Method, DbName, Path, <<" HTTP/1.1\r\nHost: ">>, Host,
      <<"\r\nContent-Type: application/json; charset=utf-8\r\n">>,
      spellHeaders(NewHeaders), <<"\r\n">>, Body
   ];
request(true, Body, Method, Host, _DbName, Path, Headers) ->
   ContentLength = integer_to_binary(iolist_size(Body)),
   NewHeaders = [{<<"Content-Length">>, ContentLength} | Headers],
   [
      Method,  <<"/_db/_system">>, Path, <<" HTTP/1.1\r\nHost: ">>, Host,
      <<"\r\nContent-Type: application/json; charset=utf-8\r\n">>,
      spellHeaders(NewHeaders), <<"\r\n">>, Body
   ].